import matplotlib.pyplot as plt   #nacteni knihovny pro kresleni, instalace pomocí příkazu: pip install matplotlib
import numpy as np    #nacteni numericke knihovny numpy, instalace pomocí příkazu: pip install numpy
from lmfit import Minimizer, Parameters, fit_report   #nacteni knihovny pro fitovani LMfit, instalace pomocí příkazu: pip install numpy

#nacteni dat do matice
data=np.loadtxt('Data.dat')  
xdata=data[:,0]
ydata=data[:,1]

def gaussian(x,p,w,g):
    return p/(g*2*np.pi) * np.exp(-(x-w)**2/(2*g**2))

#definice residualu: rozdilu fitovane funkce a dat
def residual(par, xdata, ydata):
    teor=gaussian(xdata, par['p'], par['w'],par['g']) 
    return teor-ydata    

#vytvoreni parametru a jejich startovacich hodnot. Mozno nastavit, zda se parametr fituje (vary=True) nebo ne 
pars = Parameters()
pars.add('p', value=7589,vary=True)
pars.add('w', value=2,vary=True)
pars.add('g', value=0.06597753,vary=True)

OriginalModel= ydata+residual(pars,xdata,ydata)  # calculate the original model spectrum

#fitovani
minner = Minimizer(residual, pars, fcn_args=(xdata,ydata))
result = minner.minimize(method='leastsq')
FinalModel = ydata + result.residual   # calculate final fit spectrum: data + residual
print(fit_report(result))  # write error report, alternative result.params.pretty_print()
#ulozeni statistiky do souboru
FileFit = open('Statistika.dat', 'w')
print (fit_report(result), file=FileFit)

#ulozeni nafitovaneho spektra do souboru
FileData= open('Spektrum.dat', 'w')
for i in range(len(xdata)):
    print (xdata[i], "\t",FinalModel[i], file=FileData)

# vykresleni fitu na monitor
plt.plot(xdata, OriginalModel, 'g-', label='Original model')
plt.plot(xdata, ydata, 'k-', label='data')
plt.plot(xdata, FinalModel, 'r-', label='Fitted model')
legend = plt.legend(loc='upper right', shadow=False, fontsize='x-small')
plt.show()

