import matplotlib.pyplot as plt   #nacteni knihovny pro kresleni, instalace pomocí příkazu: pip install matplotlib
import numpy as np    #nacteni numericke knihovny numpy, instalace pomocí příkazu: pip install numpy
from lmfit import Minimizer, Parameters, fit_report   #nacteni knihovny pro fitovani LMfit, instalace pomocí příkazu: pip install numpy

#load the data to mattrix data
data=np.loadtxt('Data.dat')  
xdata=data[:,0]  # data of wavenumber
ydata=data[:,1]  # reflectivity data

#theoretical calculation of reflectivity
def Teor(p, x):  #teoretická fitovací funkce, v p jsou "parametry" a v x osa x
    e = p['eINF']-p['w']**2/(x*(x+1j*p['g']))  #dielektricka funkce
    N = e**0.5      #index lomu
    R=abs((N-1)/(N+1))**2 #vypocet odrazivosti
    return R    

# definition of residual function : difference between theory and experiment
def residual(params, x, ydata):
	return Teor(params, x)-ydata


#creatin of parameters and settin their starting values: in nonlinear fit should not be far from the end result. 
#it is possible to set whether the parameter is fitted or not via (vary=True, vary=False) 
StartParams = Parameters()
StartParams.add('eINF', value=9,vary=True)
StartParams.add('w', value=1000,vary=True)
StartParams.add('g', value=100,vary=True)
original= Teor(StartParams,xdata)  #calculate original spectrum : in nonlinear fit should not be far from data 


#fitting
minner = Minimizer(residual, StartParams, fcn_args=(xdata,ydata))
result = minner.minimize(method='leastsq')
print(fit_report(result))  # write error report, alternative result.params.pretty_print()
FitParams=result.params  #slovnik s objekty Parameter s nafitovanymi hodnotami parametru a jejich chybami. 
final = Teor(FitParams,xdata) # calculate final fitted spectrum 

#for following work with fitted parameters can be helpful:
print ()
print ("hodnota eINF: ",  FitParams['eINF'].value, " a jeho chyba",  FitParams['eINF'].stderr)
print ("hodnoty korelacnich koeficientu s dalsimi parametry: ", FitParams['eINF'].correl)


#ulozeni nafitovaneho spektra do souboru
FileData= open('Spektrum.dat', 'w')
for i in range(len(xdata)):
    print (xdata[i], "\t",final[i], file=FileData)

#ulozeni statistiky do souboru
FileFit = open('Statistika.dat', 'w')
print (fit_report(result), file=FileFit)


# vykresleni fitu na monitor
plt.plot(xdata, original, 'g-', label='Model spectrum with start parameters')
plt.plot(xdata, ydata, 'k-', label='data')
plt.plot(xdata, final, 'r-', label='Model spectrum with fitted parameters')
legend = plt.legend(loc='upper right', shadow=False, fontsize='x-small')
plt.show()

